"""
To maintain compatibility b/w py2 & py3
"""
import sys
PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3
if PY2:
    from urllib import urlopen
elif PY3:
    from urllib.request import urlopen


import re
from django.conf import settings
from django import template
from django.template.base import TemplateSyntaxError
from django.template.base import kwarg_re
register = template.Library()

from img_cache.helpers import ImageResize
from img_cache import default

class ImageCacheNode(template.Node):
    def __init__(self, nodelist, src, kwargs, asvar=None):
        self.nodelist = nodelist
        self.src = src
        self.kwargs = kwargs
        self.asvar = asvar
    def render(self, context):
        for key,val in self.kwargs.items():
            self.kwargs[key] = val.resolve(context)

        self.src = self.src.resolve(context)
        img = ImageResize(self.src, **self.kwargs).resize()
        if self.asvar:
            context.push()
            context[self.asvar] = img
            output = self.nodelist.render(context)
            context.pop()
        else:
            output = img
        return output

@register.tag(name='imgcache')
def imgcache(parser, token):
    bits = token.split_contents()
    if len(bits) < 2:
        raise TemplateSyntaxError("'%s' takes at least one argument,a src url of img" % bits[0])
    src = parser.compile_filter(bits[1])
    asvar = None
    bits = bits[2:]
    if len(bits) >= 2 and bits[-2] == "as":
        asvar = bits[-1]
        bits = bits[:-2]
    kwargs = {}
    if len(bits):
        for bit in bits:
            match = kwarg_re.match(bit)
            if not match:
                raise TemplateSyntaxError("Malformed arguments to imgcache")
            name, value = match.groups()
            if name:
                kwargs[name] = parser.compile_filter(value)
    nodelist = parser.parse(('endimgcache',))
    parser.delete_first_token()
    return ImageCacheNode(nodelist, src, kwargs, asvar)

@register.filter(name='imgcache_src')
def imgcache_src(src, q=10):
    try:
        q = int(q)
    except Exception:
        raise TemplateSyntaxError("resize percent should be a intger number enclosed with single quotes!")
    return ImageResize(src, q=int(q)).resize().base64

@register.filter(name='imgcache_content')
def imgcache_content(content, q=10):
    DEFAULT_CONTENT_CLASS = getattr(settings,'IMGCACHE_DEFAULT_CONTENT_CLASS', default.IMGCACHE_DEFAULT_CONTENT_CLASS )
    DEFAULT_CONTENT_STYLE = getattr(settings,'IMGCACHE_DEFAULT_CONTENT_STYLE', default.IMGCACHE_DEFAULT_CONTENT_STYLE )
    img_replacement = u"""
    <img src="{0}" data-src="{1}" class="{2}" style="{3}" alt="{4}">
    """
    img_pattern = r'(<img [^>]+>)'
    img_src_pattern = r'src="([^"]+)"'
    img_height_pattern = r'height="/d+'
    img_width_pattern = r'width="/d+"'
    img_alt_pattern = r'alt="([^"]+)"'
    img_tags = re.findall(img_pattern, content)
    for img in img_tags:
        try:
            img_src = re.findall(img_src_pattern, img)[0]
            try:
                img_alt = re.findall(img_alt_pattern, img)[0]
            except Exception as NoImgAlt:
                img_alt = 'image-preview'
            try:
                img_height = re.findall(img_height_pattern, img)[0]
                img_width = re.findall(img_width_pattern, img)[0]
            except Exception as NoWidthHeight:
                img_height = 377
                img_width = 727
        except Exception as NoImgSrc:
            img_src = None
        if img_src:
            img_data = imgcache_src(img_src, q)
            new_img_repl = img_replacement.format(img_data, img_src,DEFAULT_CONTENT_CLASS, DEFAULT_CONTENT_STYLE, img_alt)
            content = content.replace(img, new_img_repl)
    return content



IMGCACHE_DEFAULT_STYLE='width:100%' # used in imgcache and imgcache_src
IMGCACHE_DEFAULT_CLASS='img-responsive'
IMGCACHE_DEFAULT_Q='10' # default resizing percent
IMGCACHE_TMP_DIR='img-cache' # full path will be /tmp/img-cache
IMGCACHE_KEY_PREFIX='django-img-cache' # cache_key_prefix for saving img
IMGCACHE_DEFAULT_CONTENT_CLASS='img-blur' # used in imgcache_content
IMGCACHE_DEFAULT_CONTENT_STYLE='width:100%' # used in imgcache_content


from __future__ import print_function
import os
import tempfile
import json
import base64
from django.conf import settings
from django.core.cache import cache
from PIL import Image

from img_cache.compat import urlopen
from img_cache import default

SITE_URL = getattr(settings, 'SITE_URL', 'http://localhost:8000')
TMP_DIR = tempfile.gettempdir() + '/' + getattr(settings, 'IMGCACHE_TMP_DIR', default.IMGCACHE_TMP_DIR)
CACHE_KEY_PREFIX = getattr(settings,'IMGCACHE_KEY_PREFIX', default.IMGCACHE_KEY_PREFIX)
class ImageResize(object):
    """
    helper class to deal with image resizing
    """
    def __init__(self, url, **kwargs):
        self.width = kwargs.get('width', None)
        self.height = kwargs.get('height', None)
        self.q = kwargs.get('q', None)
        # if image is not in some bucket(s3 etc.) then fetch from http://localhost/img-src
        self.url = url if 'http' in url else SITE_URL + url
        self.name = url.split('/')[-1]
        self.format = self.name.split('.')[-1]
        self.base64 = None
        if self.q:
            self.cache_key = CACHE_KEY_PREFIX + '-' + self.url + str(self.q)
        else:
            self.cache_key = CACHE_KEY_PREFIX + '-' + self.url + str(self.width) + 'x' + str(self.height)

    def _encode(self):
        data = {'width':self.width, 'height': self.height, 'base64':self.base64, 'name':self.name,
                'url':self.url, 'format':self.format, 'q':self.q, 'cache_key':self.cache_key
            }
        return json.dumps(data)

    def _decode(self):
        if cache.get(self.cache_key):
            data = json.loads(cache.get(self.cache_key))
        else:
            data = {}
        self.width = data.get('width')
        self.height = data.get('height')
        self.base64 = data.get('base64')
        self.format = data.get('format')
        self.url = data.get('url')
        self.q = data.get('q')
        self.name = data.get('name')
        return self

    def resize_calculator(self, width, height, q):
        resized_width = round((width*q*1.0)/100.0)
        resized_height = round((height*q*1.0)/100.0)
        return int(resized_width),int(resized_height)

    def is_img_cached(self):
        if self.cache_key and cache.get(self.cache_key):
            return True
        return False

    def resize(self):
        if self.is_img_cached():
            return self._decode()
        if not os.path.exists(TMP_DIR):
            os.mkdir(TMP_DIR)
        img_tmp_file = '{0}/{1}'.format(TMP_DIR, self.name)
        with open(img_tmp_file, 'wb') as im:
            im.write(urlopen(self.url).read())
        large_image = Image.open(img_tmp_file)

        if self.q:
            if not (self.q > 1 and self.q < 100):
                raise Exception("resize percent should be b/w 1 and 100")
            self.width, self.height = self.resize_calculator(large_image.width, large_image.height, self.q)
        
        if not (self.width and self.height):
            raise Exception("please specify height and width or resize percent")
        
        resized_img = large_image.resize((self.width, self.height), Image.ANTIALIAS)
        resized_img_file = "{0}/tmp_{1}".format(TMP_DIR, self.name)
        resized_img.save(resized_img_file)
        with open(resized_img_file,"rb") as im:
            self.base64 = "data:image/{0};base64,{1}".format(self.format, base64.b64encode(im.read()))
        try:
            os.remove(img_tmp_file)
            os.remove(resized_img_file)
        except Exception as e:
            print(e)
            print("Unable to remove tmp imgs file in {0}".format(TMP_DIR))
        cache.set(self.cache_key, self._encode())
        return self

    def __str__(self):
        return """
        <img src="{0}" width="{1}" height="{2}" data-src="{3}">
        """.format(self.base64, self.width, self.height, self.url)
    
    def __repr__(self):
        return self.__str__()


import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()
# os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-img-cache',
    version='0.1',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description='A simple django app for faster image caching',
    long_description=README,
    url='https://prdpx7.github.io/',
    author='Pradeep Khileri',
    author_email='pradeepchoudhary009@gmail.com',
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        "Pillow",
        "Django>=1.10"
    ],
)